/*
 * Evolution.cpp
 *
 *  Created on: 2013-03-17
 *      Author: Kamil
 */




#include "Evolution.h"

// Constructor runs pre-loop
// Initialize population, open an output file stream
// Set the current generation and the maximum number of generations
Evolution::Evolution()
{
	P = new Population(defaultsize, defaultindsize);
	currentloop = 0;
	maxloops = defaultmaxloops;
	expired = false;
	outstream = new ofstream("fitness.csv", ios::out);
		if(outstream->fail())
		{
			cout << endl << "Could not open fitness.csv for writing" << endl;
			exit(1);
		}
	*outstream << "Generations,Best Fitness,Mean Fitness" << endl;
}

// A constructor with specified parameters
Evolution::Evolution(int size, int indsize, int loops)
{
	P = new Population(size, indsize);
	currentloop = 0;
	if(loops > currentloop)
		maxloops = loops;
	else
		maxloops = defaultmaxloops;
	expired = false;
	outstream = new ofstream("fitness.csv", ios::out);
		if(outstream->fail())
		{
			cout << endl << "Could not open fitness.csv for writing" << endl;
			exit(1);
		}
	*outstream << "Generations,Best Fitness,Mean Fitness" << endl;
}

// Delete dynamically allocated memory
Evolution::~Evolution()
{
	delete P;
}

// This is the meat of the evolution class
void Evolution::Loop()
{
	int ssize = 0;
	double disqpc = 0.1;
	cout << endl << P->info() << endl << endl;

	// Run the loop as long as the termination condition(s) have not been met
	while(!P->solution() && !expired)
	{
		// Output to the screen and the file
		outputCSV();
		outputScreen();

		// Disqualify the necessary number of Individuals from the Population
		P->disqualify(disqpc * (double) currentloop);
		P->setMR(1.0 - P->stddev());
		P->setPR(0.5 * (1.0 - P->stddev()));

		// Call the selection method for parent selection
		ssize = (int) (0.2 * P->size());
		P->select(ssize, true);

		// Generate new offspring
		P->newOffspring(true);

		// Call the selection method for survivor selection
		ssize = (int) (0.2 * P->size());
		P->select(ssize, true);

		// Increment the loop number (i.e., number of generations)
		currentloop++;
		if(currentloop >= maxloops)
			expired = true;
	}

	// When the loop is done, print some relevant information to the screen
	cout << endl << P->info() << endl << endl;
	outstream->close();
}

// Method to output a line to the CSV file
void Evolution::outputCSV()
{
	*outstream << currentloop << "," << P->fittest().fitness() << "," << P->fitness() << endl;
}

// Method to output a line to the screen
void Evolution::outputScreen()
{
	cout << "Generation: " << currentloop << "\tAge: " << P->age() << "\tSize: " << P->size() << "\tMean Fitness: " << P->fitness() << "\tBest Fitness: " << P->fittest().fitness() << "\tSD: " << P->stddev() << endl;
}






