/*
 * Individual.h
 *
 *  Created on: 2013-03-15
 *      Author: Kamil
 */

#ifndef INDIVIDUAL_H_
#define INDIVIDUAL_H_

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

#ifndef M_E
#define M_E 2.71828182845904523536
#endif

#define TOLERANCE 0.00001
#include <random>
#include <iostream>
#include <string>
#include <sstream>
#include <chrono>
#include <cmath>
#include <vector>
using namespace std;

class Individual
{
private:
	vector<double> _genotype;
	double _fitness;
	double _mrate;
	double _prate;
	double _sdfactor;
	bool _elite;
	int _age;
	static const int defaultMaxSize = 20;

	int setSize(int);
	void setElite();

public:
	Individual(int size = defaultMaxSize, double mrate = 0.5, double prate = 0.7);
	Individual(const Individual&);
	Individual(const Individual*);
	~Individual();

	Individual& operator=(const Individual&);
	Individual* operator=(const Individual*);

	void setFitness();
	friend void operator!(Individual&);
	void setMR(double);
	void setPR(double);
	void setSDF(double);

	double fitness();
	double mutationRate();
	double permutationRate();
	double sdFactor();
	int size();
	int age();
	bool isElite();
	double allele(int);
	string info();

	friend vector<Individual> operator*(Individual&, Individual&);
	friend vector<Individual> operator+(Individual&, Individual&);
	friend class Population;
};

#endif /* INDIVIDUAL_H_ */
