/*
 * Population.h
 *
 *  Created on: 2013-03-16
 *      Author: Kamil
 */

#ifndef POPULATION_H_
#define POPULATION_H_

#include "Individual.h"
#include <string>
#include <sstream>
#include <random>

using namespace std;

class Population
{
private:
	vector<Individual> pool;
	vector<Individual> elites;

	double _age;
	double _fitness;
	double _tfitness;
	double _stddev;
	double _rsd;

	double fprank(Individual&);

	int setSize(int);
	void setElite(Individual&);
	void roulette();
	void tournament(int);

	static const int defaultMaxSize = 1000;
	static const int defaultIndSize = 10;
	double tolerance;

public:
	Population(int size = defaultMaxSize, int indsize = defaultIndSize);
	Population(const Population&);
	~Population();

	Population& operator=(const Population&);

	void disqualify(double);
	void update();
	void mutate();
	void mutate(int, int);
	void crossover(bool lc = false);
	void crossover(int, int, bool lc = false);
	void newOffspring(bool lc = false);
	void newOffspring(int, int, bool lc = false);
	void select(int samplesize = -1, bool ts = false);
	void setMR(double);
	void setMR(double, int, int);
	void setPR(double);
	void setPR(double, int, int);
	void setFactor(double);
	void setFactor(double, int, int);

	Individual& fittest(vector<Individual>);
	Individual& fittest();
	double fitness();
	int size();
	double age();
	double stddev();
	double rsd();
	int numElites();
	bool isElite(Individual&);
	bool solution();
	string info();
};

#endif /* POPULATION_H_ */
